// See LICENSE.SiFive for license details.

package sifive.fpgashells.devices.xilinx.xilinxml507mig

import Chisel._
import chisel3.core.{Input, Output}
import chisel3.experimental.Analog
import freechips.rocketchip.config.Field
import freechips.rocketchip.diplomacy._
import freechips.rocketchip.subsystem.BaseSubsystem

case object MemoryML507Key extends Field[XilinxML507MIGParams]

trait HasMemoryML507 { this: BaseSubsystem =>
    val memory = LazyModule(new XilinxML507MIG(p(MemoryML507Key)))

    memory.node := memBuses.head.toDRAMController(Some("xilinxml507mig"))()
}

class MemorySysIO extends Bundle {
    val clk0 = Input(Clock())
    val clk90 = Input(Clock())
    val clkdiv0 = Input(Clock())
    val clk_locked = Input(Bool())
    val clk_idelay = Input(Clock())
    val reset = Input(Bool())
}

class MemoryDDR2IO extends Bundle {
    val dq = Analog(64.W)
    val a = Output(Bits(13.W))
    val ba = Output(Bits(2.W))
    val ras_n = Output(Bits(1.W))
    val cas_n = Output(Bits(1.W))
    val we_n = Output(Bits(1.W))
    val cs_n = Output(Bits(1.W))
    val odt = Output(Bits(1.W))
    val cke = Output(Bits(1.W))
    val dm = Output(Bits(8.W))
    val dqs = Analog(8.W)
    val dqs_n = Analog(8.W)
    val ck = Output(Bits(2.W))
    val ck_n = Output(Bits(2.W))
}

trait HasMemoryML507Bundle {
    val ddr_sys: MemorySysIO
    val ddr2: MemoryDDR2IO
}

trait HasMemoryML507ModuleImp extends LazyModuleImp with HasMemoryML507Bundle {
    val outer: HasMemoryML507
    val ddr_sys = IO(new MemorySysIO)
    val ddr2 = IO(new MemoryDDR2IO)

    ddr_sys <> outer.memory.module.io.port_sys
    ddr2 <> outer.memory.module.io.port_ddr2
}
